// A 3D simulation of Rubik's Cube using OpenGL.
// Copyright (C) 2002  Martin Geisler <gimpster@gimpster.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef RUBIKSCUBE_H
#define RUBIKSCUBE_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <GL/glut.h>
#include "RoundedCube.h"

/** A Rubik's Cube. This class manages the 27 cubes that make up a
    Rubik's Cube. */
class RubiksCube {
 private:
  RoundedCube *cubes[3][3][3];
  int rot_x[3], rot_y[3], rot_z[3];
  int total_x, total_y, total_z;
 public:
  /** The constructor. Memory will be allocated for the 27 cubes. */
  RubiksCube();

  /** Destructs the 27 cubes. */
  ~RubiksCube();

  /** Renders the cubes. This will make the necessary calls to OpenGL
   *  to render the 27 cubes. You shouldn't call render() between
   *  calls to glBegin() ... glEnd(). */
  void render();

  /** Rotates the designated block around the x-axis. */
  void rotateX(unsigned int block, int degrees);
  /** Rotates the designated block around the y-axis. */
  void rotateY(unsigned int block, int degrees);
  /** Rotates the designated block around the z-axis. */
  void rotateZ(unsigned int block, int degrees);
};

#endif
