// A 3D simulation of Rubik's Cube using OpenGL.
// Copyright (C) 2002  Martin Geisler <gimpster@gimpster.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <iostream>
// What does this do? Well, it makes things compile using Borland C++Builder.
#define GLUT_BUILDING_LIB
#include <GL/glut.h>
#include <math.h>
#include "RoundedCube.h"
#include "RubiksCube.h"
#include "commandoptions.h"

bool rotate    = false;
bool lighting  = true;
bool wireframe = false;

float rot_speed = 1;
RubiksCube cube;
int block = 0;
int rot_x, rot_y;

//---------------------------------------------------------------------------
void reshape(int w, int h) {
  float aspect;
  if (h != 0)
    aspect = (float)w/(float)h;
  else
    aspect = 1;

  glViewport(0, 0, w, h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  gluPerspective(45, aspect, 1, 100);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glTranslatef(0, 0, -8);

}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void display() {
  static float time = 0;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glPushMatrix();
    glRotatef(rot_x, 1, 0, 0);
    glRotatef(rot_y, 0, 1, 0);
    glRotatef(time, 1.2, 0.9, 0.2);
    cube.render();
  glPopMatrix();

  glFlush();
  glutSwapBuffers();

  if (rotate) {
    glutPostRedisplay();
    time += rot_speed;
  }
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void special_keyboard(int key, int x, int y) {

  switch (key) {
  case GLUT_KEY_UP:
    rot_x = (rot_x - 5) % 360;
    break;
  case GLUT_KEY_DOWN:
    rot_x = (rot_x + 5) % 360;
    break;
  case GLUT_KEY_LEFT:
    rot_y = (rot_y - 5) % 360;
    break;
  case GLUT_KEY_RIGHT:
    rot_y = (rot_y + 5) % 360;
    break;
  }

  glutPostRedisplay();

}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void keyboard(unsigned char key, int x, int y) {

  switch (key) {
  case '1':
    block = 0;
    break;
  case '2':
    block = 1;
    break;
  case '3':
    block = 2;
    break;
  case 'u':
    cube.rotateX(block, -3);
    break;
  case 'j':
    cube.rotateX(block, 3);
    break;
  case 'i':
    cube.rotateY(block, -3);
    break;
  case 'k':
    cube.rotateY(block, 3);
    break;
  case 'o':
    cube.rotateZ(block, -3);
    break;
  case 'l':
    cube.rotateZ(block, 3);
    break;
  case 'w':
    wireframe = !wireframe;
    if (wireframe) {
      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    } else {
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    }
    break;
  case 'e':
    lighting = !lighting;
    if (lighting) {
      glEnable(GL_LIGHTING);
    } else {
      glDisable(GL_LIGHTING);
    }
    break;
  case 'r':
    rotate = !rotate;
    std::cerr << "rotate: " << rotate << std::endl;
    break;
  case 'z':
    rot_speed += 0.05;
    std::cerr << "rot_speed: " << rot_speed << std::endl;
    break;
  case 'Z':
    rot_speed -= 0.05;
    std::cerr << "rot_speed: " << rot_speed << std::endl;
    break;
  case 'q':
  case 27: /* Escape */
    exit(0);
  }

  glutPostRedisplay();

}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void idle() {
  if (rotate)
    glutPostRedisplay();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void glInit() {

  GLfloat light_ambient[]  = { 0.25, 0.25, 0.25, 1.0 }; // faint white light
  GLfloat light_diffuse[]  = { 1.0, 1.0, 1.0, 1.0 }; // white diffuse
  GLfloat light_specular[] = { 1.0, 1.0, 1.0, 1.0 }; // highlights
  GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };

  glClearColor(0.5, 0, 0.8, 0);
  glShadeModel(GL_SMOOTH);

  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);

  glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);

  glEnable(GL_LIGHT0);
  if (lighting)
    glEnable(GL_LIGHTING);

  if (wireframe)
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  else
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);


  glEnable(GL_DEPTH_TEST);

}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int main(int argc, const char *argv[]) {

  commandoptions c;
  /*
  c.register_flag(rotate, "rotate", 'r', "Makes the cube rotate");
  c.register_flag(lighting, "lighting", 'l', "Disable lightning");
  c.register_flag(wireframe, "wireframe", 'w', "Use wireframe when drawing");
  */
  try {
    c.process_command_line(argc, argv);
  } catch (commandoptions_error &e) {
    std::cerr << e.what() << std::endl;
    return 1;
  }

  glutInit(&argc, const_cast<char **>(argv));

  glutInitWindowPosition(50, 50);
  glutInitWindowSize(400, 400);

  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);

  glutCreateWindow("A Rubik's Cube");
  glutSetIconTitle("rubik.ico");

  glInit();

  glutReshapeFunc(reshape);

  glutDisplayFunc(display);

  glutKeyboardFunc(keyboard);
  glutSpecialFunc(special_keyboard);

  //glutIdleFunc(idle);

  glutMainLoop();

  return 0;

}
//---------------------------------------------------------------------------

